import numpy as np
import cv2
from skimage import io
from picamera import PiCamera
from picamera.array import PiRGBArray
from time import sleep

resolution = (320, 240)
framerate = 32
image_out_path = "images/out.jpeg"
model_path = "models/MobileNetSSD_deploy.caffemodel"
prototxt_path = "models/MobileNetSSD_deploy.prototxt"
min_confidence = 0.4
CLASSES = ["background", "aeroplane", "bicycle", "bird", "boat",
    "bottle", "bus", "car", "cat", "chair", "cow", "diningtable",
    "dog", "horse", "motorbike", "person", "pottedplant", "sheep",
    "sofa", "train", "tvmonitor"]

camera = PiCamera()
camera.resolution = resolution
camera.framerate = framerate
rawCapture = PiRGBArray(camera, size=resolution)
stream = camera.capture_continuous(rawCapture, format='bgr', use_video_port=True)

# wait for camera to warm up
sleep(0.1)

net = cv2.dnn.readNetFromCaffe(prototxt_path, model_path)
net.setPreferableTarget(cv2.dnn.DNN_TARGET_MYRIAD)

cv2.namedWindow("Camera Detecting")

for f in stream:

    frame = f.array
    rawCapture.truncate(0)
    frame = cv2.flip(frame, 0)
    H, W = frame.shape[:2]

    blob = cv2.dnn.blobFromImage(frame, size=(300, 300), ddepth=cv2.CV_8U)
    net.setInput(blob, scalefactor=1.0/127.5, mean=[127.5, 127.5, 127.5])

    detections = net.forward()

    for i in np.arange(detections.shape[2]):
        confidence = detections[0,0,i,2]

        if confidence < min_confidence:
            continue

        idx = int(detections[0,0,i,1])

        if CLASSES[idx] != 'person':
            continue
    
        box = detections[0,0,i,3:7]*np.array([W, H, W, H])
        startX, startY, endX, endY = box.astype("int")
    
        frame = cv2.rectangle(frame, (startX, startY), (endX, endY), (255, 0, 0), 2)

    cv2.imshow('Camera Detecting', frame)

    # Any key
    if cv2.waitKey(1) != -1:
        break


stream.close()
rawCapture.close()
camera.close()
cv2.destroyAllWindows()

