import numpy as np
import cv2
from skimage import io

image_path = "car_entering_lot_01.jpg"
image_out_path = "out.jpeg"
model_path = "MobileNetSSD_deploy.caffemodel"
prototxt_path = "MobileNetSSD_deploy.prototxt"
min_confidence = 0.4

CLASSES = ["background", "aeroplane", "bicycle", "bird", "boat",
    "bottle", "bus", "car", "cat", "chair", "cow", "diningtable",
    "dog", "horse", "motorbike", "person", "pottedplant", "sheep",
    "sofa", "train", "tvmonitor"]

net = cv2.dnn.readNetFromCaffe(prototxt_path, model_path)
net.setPreferableTarget(cv2.dnn.DNN_TARGET_MYRIAD)

frame = cv2.imread(image_path)
H, W = frame.shape[:2]

blob = cv2.dnn.blobFromImage(frame, size=(300, 300), ddepth=cv2.CV_8U)
net.setInput(blob, scalefactor=1.0/127.5, mean=[127.5, 127.5, 127.5])

detections = net.forward()

for i in np.arange(detections.shape[2]):
    confidence = detections[0,0,i,2]

    if confidence < min_confidence:
        continue

    idx = int(detections[0,0,i,1])

    if CLASSES[idx] != 'car':
        continue
    
    box = detections[0,0,i,3:7]*np.array([W, H, W, H])
    startX, startY, endX, endY = box.astype("int")
    
    frame = cv2.rectangle(frame, (startX, startY), (endX, endY), (255, 0, 0), 2)

cv2.imwrite(image_out_path, frame)
small_frame = cv2.resize(frame, (W // 8,H // 8))
cv2.imshow('Image', small_frame)
cv2.waitKey(0)

cv2.destroyAllWindows()

