
class TrackableObject:
    """
    A trackable object based on the one by pyimagesearch
    """
    def __init__(self, objectID, centroid, decay=0.8):
        """
        Creates a new trackable object with no movement information
        
        Arguments:
        objectID - numeric ID for keeping track of objects
        centroid - the most recent location of the object as (x,y)
        decay - OPTIONAL - the moving average decay rate use in 
              calculating the direction vector.

        """
        self.objectID = objectID
        self.centroid = centroid
        self.count = 1
        self.vector = (0,0)
        self.counted = False
        self.decay = decay


    def update_centroid(self, centroid):
        """
        Updates the current location of the object. It also
        updates the direction vector via a moving exponential
        average.
        
        Arguments:
        centroid - a tuple containing the location as (x,y)
        
        """
        x,y = centroid
        old_x, old_y = self.centroid
        old_dx, old_dy = self.vector
        # running exponential average vector
        self.vector = (old_dx + self.decay * (x - old_dx),
                       old_dy + self.decay * (y - old_dy))
        self.centroid = centroid
        self.count += 1

