# Device Setup
- ## Hardware:
  - Rasberry Pi 4b 8 GB
  - Intel Movidius Neural Compute Stick 2 (NCS2)
  - 32 GB micro USB
  - Rasberry NoIR Camera

- ## Software:
  - 2021-05-07-raspios-buster-armhf-full.img
  - Raspberry: imager_1.6.2.exe
  - VNC-Viewer-6.21.406-Windows.exe
  - l_openvino_toolkit_runtime_raspbian_p_2021.4.689.tgz
- ## Steps:
  1. Download OS and install using the imager onto the micro SD.
  2. Add an empty file named "ssh" to the root directory of the OS on the micro SD
  3. Add a file named "wpa_supplicant.conf" in the root directory with the following code: 
        ```
        country=US
        ctrl_interface=DIR=/var/run/wpa_supplicant GROUP=netdev
        update_config=1
        network={
          scan_ssid=1
          ssid="your_wifi_ssid"
          psk="your_wifi_password"
        }
        ```
     *note: ssid is your networks name
  4. Connect all Rasberry Pi components except for the NCS2 and plug it in.
  5. SSH into:
        ```
        Host: raspberrypi (or its ip)
        User: pi
        Port: 22
        password: rasberry
        ```
  6. Type ```passwd``` to change the password.
  7. In the terminal type: ```sudo raspi_config```
  8. Select: _Interface Options_ -> _VNC_ -> _YES_ -> _OK_
  9. Select: _Display_ -> _Resolution_ -> Anything but default -> _OK_
  10. In the terminal type ```sudo reboot``` to restart the system
  11. Run RealVNC's VNC Viewer
  12. Create new connection:
        ```
        VNC Server: raspberrypi.local (or ip)
        ```
      Then click _OK_
  13. Double click the connection icon. Login using:
        ```
        user: pi
        password: your_password
        ```
  14. Complete the Raspberry Pi Setup using the GUI wizard.
      - Make sure to update the software
      - reboot when finished
  15. Download [OpenVino](http://storage.openvinotoolkit.org/repositories/openvino/packages/2021.4.1/l_openvino_toolkit_runtime_raspbian_p_2021.4.689.tgz)
  16. Install OpenVino
      - In the terminal type:
           ```
           cd ~/Downloads/
           sudo mkdir -p /opt/intel/openvino_2021
           sudo tar -xf l_openvino_toolkit_runtime_raspbian_p_2021.4.689.tgz -- strip 1 -C /opt/intel/openvino_2021
           ```
  17. Install dependencies by typing: ```sudo apt install cmake```
  18. Set up environment variables:
      - Type:
           ```
           source /opt/intel/openvino_2021/bin/setupvars.sh
           echo "source /opt/intel/openvino_2021/bin/setupvars.sh" >> ~/.bashrc
           echo "source $HOME/.bash_profile" >> ~/.bashrc
           ```
           *you may need to type the first line again every time you open a new terminal. The second line is supposed to avoid this behavior but doesn't always work.
      - Open new terminal. It should say that the environment was initialized.
  19. Add USB rules:
      - Add current user to "users" group by typing:
           ```
           sudo usermod -a -G users "$(whoami)"
           ```
      - Type:
           ```
           sh /opt/intl/openvino_2021/install_dependencies/install_NCS_udev_rules.sh
           ```
  20. Plug in the NCS2 into a USB3 port (blue).
  21. Install dependencies by typing:
       ```
       python3 -m pip install -r requirements.in
       pip install imutils
       pip install Cython
       python3 -m pip install -U pip
       sudo apt-get install libatlas-base-dev
       ```
  22. Install virtual environment
       ```
       sudo pip3 install virtualenv virtualenvwrapper
       export WORKON_HOME=~/Projects/Environments
       echo "WORKON_HOME=~/Projects/Environments" >> ~/.bashrc
       mkdir -p $WORKON_HOME
       source /usr/local/bin/virtualenvwrapper.sh
       echo "source /usr/local/bin/virtualenvwrapper.sh" >> ~/.bashrc
       mkvirtualenv upark-sensor
       pip3 install dlib
       pip3 install numpy
       pip3 install scikit-image
       pip3 install picamera
       pip3 install -U pytest
       pip3 install pytomlpp
       ```
  23. Enable the camera
      - Type ```sudo raspi-config```
      - Click _Interface Options_ -> _Camera_ -> _YES_ -> _Finish_
      - Reboot the pi
  24. Correct NoIR camera hue
      - Type ```raspistill -awb greyworld```
      - Add ```awb_auto_is_greyworld=1``` to the file /boot/config.txt
  25. Set up the sensor to auto start on reboot.
      - Type ```crontab -e```
      - Select prefered editor.
      - Add the following below the comments. Make sure to end with a newline character (return).
      ```
      SHELL=/bin/bash
      PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin
      WORKON_HOME=/home/pi/Projects/Environments
      PROJECT_HOME=/home/pi/Projects/UPark-Device
      DISPLAY=:0.0
      MAILTO=youremail@domain.com
      
      @reboot /usr/bin/echo "test" &> /tmp/test.log
      @reboot cd /home/pi/Projects/UPark-Device/ && /bin/bash -l /home/pi/Projects/UPark-Device/autorun.sh &> /tmp/upark_autorun.log

      ```
  26. Install Telegraf for sensor hardware metric collection.
      - Download and install [Telegraf](https://dl.influxdata.com/telegraf/releases/telegraf_1.21.4-1_armhf.deb)
      - Configure Telegraf to collect system metrics:
        - Navigate to https://grafana.com/grafana/dashboards/928 and scroll to the bottom.
        - Copy the sample Telegraf configuration file and save it to `/etc/telegraf/telegraf.conf`.
        - Replace `your_host` in `urls = ["http://your_host:8086"]` under the `[[outputs.influxdb]]` section with the address of your InfluxDB instance.
        - Set the `hostname` field in the `[agent]` section to `sensorX`, i.e. the api username of the sensor.
      - Configure Telegraf to start at boot by adding `@reboot telegraf &` to the end of the `crontab` file, keeping the newline at the end of the file.

