from picamera import PiCamera
from picamera.array import PiRGBArray
import cv2
import numpy as np
from threading import Thread

class CameraStream:
    """
    An iterable stream that takes input from the raspberry pi's camera
    """


    def __init__(self, resolution, framerate):
        
        self.camera = PiCamera()
        self.camera.resolution = resolution
        self.camera.framerate = framerate
        #self.camera.exposure_mode = 'auto'
        #self.camera.shutter_speed = 1_000_000

        self.rawCapture = PiRGBArray(self.camera, size=resolution)
        self.stream = self.camera.capture_continuous(self.rawCapture, format='bgr', use_video_port=True)
        
        self.frame = np.zeros((*resolution,3), np.uint8)
        self.stopped = False

    
    def get_resolution(self):
        return self.camera.resolution

    def start(self):
        Thread(target=self._update, daemon=True).start()
        return self


    def _update(self):

        for f in self.stream:
            self.frame = cv2.flip(f.array, -1)
            self.rawCapture.truncate(0)
            
            if self.stopped:
                self.stream.close()
                self.rawCapture.close()
                self.camera.close()
                return

    def capturing(self):
        return not self.stopped

    def get_frame(self):
        return self.frame


    def stop(self):
        self.stopped = True


    def __iter__(self):
        return self


    def __next__(self):
        return self.get_frame()


