from server_mediator import UParkServerMediator
from cv_sensor import CVSensor
from camera_stream import CameraStream 
from video_stream import VideoStream
import pytomlpp as toml
import argparse


def main():

    description = '''The computer vision based sensor for the UPark system.'''
    arg_parser = argparse.ArgumentParser(description=description)
    arg_parser.add_argument("-nov", 
                            "--NoVideo", 
                            action='store_true', 
                            help="Don't display the video")
    arg_parser.add_argument("-nos", 
                            "--NoServer", 
                            action='store_true', 
                            help="Don't communicate updates to the UPark server.")
    arg_parser.add_argument("-v",
                            "--Video",
                            help="Use the specified video instead of the camera.")
    
    arg_parser.add_argument("-l",
                            "--Loop",
                            action='store_true',
                            help="Only works if --Video is enabled. Sets the video to loop repeatedly.")
    args = arg_parser.parse_args()

    f = open('config.toml', 'r')
    config = toml.loads(f.read())

    mediator=None

    if not args.NoServer:
        # username = input('Enter username for device: ')
        # password = input('Enter password for device: ')
        username = config['username']
        password = config['password']
    
        # communicates with the server
        mediator = UParkServerMediator(config['api_address'], 
                                     config['lot_num'], 
                                     username, 
                                     password)

    if args.Video is None:
        stream = CameraStream(config['camera_resolution'],
                                   config['camera_framerate'])
    else:
        stream = VideoStream(args.Video, 30, args.Loop)

    sensor = CVSensor(stream, mediator, not args.NoVideo, not args.NoServer, config)
    sensor.start()




if __name__ == "__main__":
    main()

