from math_util import *
import pytest

def test_deg_to_per_line_01():
    deg = 0
    center = [0,0]
    pt1, pt2 = deg_to_perp_line(deg, center)
    assert pytest.approx(pt1[0], 0.1) == 0 and pytest.approx(pt1[1], 0.1) == 0, f"Incorrect point: {pt1}"
    assert pytest.approx(pt2[0], 0.1) == 0 and pytest.approx(pt2[1], 0.1) == -100, f"Incorrect point: {pt2}"


def test_deg_to_per_line_02():
    deg = 0
    center = [50,100]
    pt1, pt2 = deg_to_perp_line(deg, center)
    assert pytest.approx(pt1[0], 0.1) == 50 and pytest.approx(pt1[1], 0.1) == 100, f"Incorrect point: {pt1}"

    assert pytest.approx(pt2[0], 0.1) == 50 and pytest.approx(pt2[1], 0.1) == 0, f"Incorrect point: {pt2}"


def test_deg_to_per_line_03():
    deg = 180
    center = [0,0]
    pt1, pt2 = deg_to_perp_line(deg, center)
    assert pytest.approx(pt1[0], 0.1) == 0 and pytest.approx(pt1[1], 0.1) == 0, f"Incorrect point: {pt1}"

    assert pytest.approx(pt2[0], 0.1) == 0 and pytest.approx(pt2[1], 0.1) == 100, f"Incorrect point: {pt2}"



def test_deg_to_per_line_04():
    deg = 90
    center = [50,100]
    pt1, pt2 = deg_to_perp_line(deg, center)
    assert pytest.approx(pt1[0], 0.1) == 50 and pytest.approx(pt1[1], 0.1) == 100, f"Incorrect point: {pt1}"
    assert pytest.approx(pt2[0], 0.1) == 150 and pytest.approx(pt2[1], 0.1) == 100, f"Incorrect point: {pt2}"


def test_deg_to_vector():
    pt = deg_to_vector(0)
    assert pytest.approx(pt[0], 0.1) == 1 and pytest.approx(pt[1], 0.1) == 0
    pt = deg_to_vector(90)
    assert pytest.approx(pt[0], 0.1) == 0 and pytest.approx(pt[1], 0.1) == 1
    pt = deg_to_vector(180)
    assert pytest.approx(pt[0], 0.1) == -1 and pytest.approx(pt[1], 0.1) == 0
    pt = deg_to_vector(270)
    assert pytest.approx(pt[0], 0.1) == 0 and pytest.approx(pt[1], 0.1) == -1


def test_rot_vector():
    pt = rot_vector(np.array([0,1]), np.pi / 2)
    assert pytest.approx(pt[0], 0.1) == -1 and pytest.approx(pt[1], 0.1) == 0
    pt = rot_vector(np.array([0,1]), np.pi)
    assert pytest.approx(pt[0], 0.1) == 0 and pytest.approx(pt[1], 0.1) == -1
    pt = rot_vector(np.array([-1 / np.sqrt(2), 1 / np.sqrt(2)]), np.pi / 4)
    assert pytest.approx(pt[0], 0.1) == -1 and pytest.approx(pt[1], 0.1) == 0


def test_vector_to_angle_01(): 
    vector = np.array([np.sqrt(3), 1])
    assert pytest.approx(vector_to_angle(vector), 0.1) == np.pi / 6
    vector = np.array([-1 * np.sqrt(3), 1])
    assert pytest.approx(vector_to_angle(vector), 0.1) == np.pi - np.pi / 6
    vector = np.array([-1 * np.sqrt(3), -1])
    assert pytest.approx(vector_to_angle(vector), 0.1) == np.pi + np.pi / 6
    vector = np.array([np.sqrt(3), -1])
    assert pytest.approx(vector_to_angle(vector), 0.1) == 2 * np.pi - np.pi / 6

def test_vector_to_angle_02():
    vector = np.array([0, 0])
    #with pytest.raises(ValueError):
    #    vector_to_angle(vector)
    assert vector_to_angle(vector) == 0
    vector = np.array([1, 0])
    assert pytest.approx(vector_to_angle(vector), 0.1) == 0
    vector = np.array([0, 1])
    assert pytest.approx(vector_to_angle(vector), 0.1) == np.pi / 2
    vector = np.array([-1, 0])
    assert pytest.approx(vector_to_angle(vector), 0.1) == np.pi
    vector = np.array([0, -1])
    assert pytest.approx(vector_to_angle(vector), 0.1) == 3 * np.pi / 2
    

def test_relative_angle_01():
    vector1 = np.array([1,1])
    vector2 = np.array([0,1])
    assert pytest.approx(relative_angle(vector1, vector2), 0.1) == np.pi / 4
    assert pytest.approx(relative_angle(vector2, vector1), 0.1) == 2 * np.pi - np.pi / 4


def test_dist_to_line(): 
    line_pt1 = np.array([100,0])
    line_pt2 = np.array([100,200])
    centroid = np.array([50,50])
    assert dist_to_line(centroid, line_pt1, line_pt2) == -50
    centroid = np.array([150,50])
    assert dist_to_line(centroid, line_pt1, line_pt2) == +50
