import numpy as np
import pandas as pd
from server_mediator import UParkServerMediator
import pytomlpp as toml
import requests
from datetime import datetime
from time import sleep
import signal
import sys


# This script is used to simulate a network of cv image sensors accrosss a upark system.

def interupt_handler(signum, frame):
    print('Simulator exiting...')
    sys.exit()


def percent_formula(min_of_day):
    min_in_day = 1440
    emptiest_min = 180 #3am
    return 0.99 * (-np.cos(2 * np.pi * (min_of_day - emptiest_min)/min_in_day) / 2 + 0.5) + np.random.rand() * 0.01


def get_data(config):
    filename = 'sensor_network_data.csv'
    return pd.read_csv(filename)


def make_mediators(config):
    data = get_data(config)
    mediators = {}
    for tup in data.itertuples():
        mediators[tup.lotid] = UParkServerMediator(config['api_address'], tup.lotid, tup.name, tup.password)
    
    return mediators


def get_passwords(config):
    data = get_data(config)
    passwords = {}
    for tup in data.itertuples():
        passwords[tup.lotid] = (tup.name, tup.password)
    return passwords


def get_capacity(config, token, lotid):
    return requests.get(config['api_address'] + f'/lot/{lotid}/capacity/', headers={"Authorization" : f"Bearer {token}"}).json()


def get_capacities(config, passwords):
    data = get_data(config)
    capacities = {}
    # get capacities from api
    for tup in data.itertuples():
        name, password = passwords[tup.lotid]
        token = get_token(config, name, password)
        capacities[tup.lotid] = get_capacity(config, token, tup.lotid)
    return capacities


def get_token(config, name, password):
    response_user = requests.post(config['api_address'] + '/token/', data={"username" : name, "password" : password}).json()
    return response_user['access']


def main():
    signal.signal(signal.SIGINT, interupt_handler)
    f = open('config.toml', 'r')
    config = toml.loads(f.read())
    mediators = make_mediators(config)
    passwords = get_passwords(config)
    capacities = get_capacities(config, passwords)
    np.random.seed(54)
    variance = {e.lotid : np.random.rand() * 0.3 + 0.7 for e in get_data(config).itertuples()}

    sleep_seconds = 60

    while True:
        # min_of_day = 900 # busiest time of day
        min_of_day = datetime.now().hour * 60 + datetime.now().minute
        for lotid, mediator in mediators.items():
            percent = percent_formula(min_of_day)
            desired_usedstalls = int(percent * capacities[lotid] * variance[lotid])
            change_of_stalls = desired_usedstalls - mediator.get_used_stalls()
            mediator.change_count(change_of_stalls)
        
        sleep(sleep_seconds)




if __name__ == '__main__':
    main()