import threading
import requests
import sys

class UParkServerMediator:


    def __init__(self, api_address, lot_num, username, password):
        self.api_address = api_address
        self.username = username
        self.password = password

        self.token = self._get_token()
        self.lot_num = lot_num
        self.api_address = api_address
        

    def _get_token(self):
        try:
            response_user = requests.post(self.api_address + '/token/', 
                                          data={"username" : self.username, "password" : self.password},
                                          headers={"Connection" : "close"}).json()
        except requests.exceptions.RequestException as e:
            print('Error encountered while getting token. Exiting...')
            print(e)
            sys.exit()
        return response_user['access']


    def change_count(self, change_of_count):
        threading.Thread(target=self._count_updater,
                         args=(change_of_count,),
                         daemon=True).start()


    def _count_updater(self, amount):
        response_patch = self._patch_lot(amount)
        if 200 <= response_patch.status_code <= 299:
            # log a successful connection here
            pass
        else:
            self.token = self._get_token()
            self._patch_lot(amount)


    def _patch_lot(self, amount):
        return requests.patch(f'{self.api_address}/lotcount/{self.lot_num}/',
                              headers={"Authorization" : f"Bearer {self.token}",
                                       "Connection" : "close"},
                              data={'usedstalls' : amount})


    def _request_lotcount(self):
        return requests.get(f'{self.api_address}/lotcount/{self.lot_num}/', 
                               headers={"Authorization" : f"Bearer {self.token}",
                                        "Connection" : "close"})


    def get_used_stalls(self):
        # get current used stalls for given lot from api
        response = self._request_lotcount()

        if 400 <= response.status_code:
            self.token = self._get_token()
            response = self._request_lotcount()
        
        return response.json()['usedstalls']

