import cv2
import numpy as np
from threading import Thread
import time
from os.path import exists

class VideoStream:
    """
    An iterable stream that takes input from a video file
    """


    def __init__(self, filename, framerate, loop=False):

        if not exists(filename):
            raise FileNotFoundError
        
        vid = cv2.VideoCapture(filename)

        self.frames = []
    
        while True:
            success, frame = vid.read()
            if success:
                self.frames.append(frame)
            else:
                break

        print("frames loaded") 
    
        vid.release()
        
        self.loop = loop
        self.frame_num = 0.0
        self.framerate = framerate
        self.delta_time = 0

        self.stopped = False


    def start(self):
        self.time = time.time()
        return self


    def get_resolution(self):
        return self.frames[0].shape[:2]


    def stop(self):
        self.stopped = True


    def __iter__(self):
        return self


    def __next__(self):

        new_time = time.time()
        self.delta_time = new_time - self.time
        self.time = new_time

        # x frames/sec * y sec = z frames
        elapsed_frames = self.framerate * self.delta_time
        # frame_num should be a decimal number in order to count partial frames
        self.frame_num += elapsed_frames            
        
        if self.stopped:
            raise StopIteration

        if self.frame_num >= len(self.frames):
            print("reached end of frames.")
            if not self.loop:
                del self.frames
                raise StopIteration
            else:
                self.frame_num %= len(self.frames)

        return self.frames[int(self.frame_num)]


