import cv2
import numpy as np

class VinoDetector:

    
    def __init__(self, bin_path, 
                       xml_path, 
                       min_confidence, 
                       classes,
                       target=cv2.dnn.DNN_TARGET_MYRIAD):

        self.net = cv2.dnn.readNet(xml_path, bin_path)
        
        self.net.setPreferableTarget(target)
        self.min_confidence = min_confidence
        self.classes = classes


    def detect(self, frame, 
                     class_name='vehicle', 
                     size=(300,300), 
                     scalefactor=1.0/127.5, 
                     mean=[127.5,127.5,127.5],
                     ddepth=cv2.CV_8U):
        
        H, W = frame.shape[:2]
        
        blob = cv2.dnn.blobFromImage(frame, size=size, ddepth=ddepth)
        self.net.setInput(blob, scalefactor=scalefactor, mean=mean)
        
        try:
            detections = self.net.forward()
        except:
            print('Error with network forward pass.')
            return []

        boxes = []
        
        for i in np.arange(detections.shape[2]):
            
            confidence = detections[0,0,i,2]

            if confidence < self.min_confidence:
                continue

            class_idx = int(detections[0,0,i,1])
                 
            if self.classes[class_idx] != class_name:
                continue

            box = (detections[0,0,i,3:7] * np.array([W, H, W, H])).astype('int')
            boxes.append(box)

        return boxes


